@@ -32,17 +32,21 @@ import java.security.Permission;
 import java.security.PermissionCollection;
 import java.security.Permissions;
 import java.security.PrivilegedAction;
+import java.security.PrivilegedExceptionAction;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.Enumeration;
 import java.util.HashMap;
+import java.util.HashSet;
 import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
+import java.util.Set;
 import java.util.TreeSet;
 import java.util.Vector;
 import java.util.jar.JarEntry;
 import java.util.jar.JarFile;
+import java.util.jar.Manifest;
 
 import net.sourceforge.jnlp.DownloadOptions;
 import net.sourceforge.jnlp.ExtensionDesc;
@@ -138,10 +142,13 @@ public class JNLPClassLoader extends URLClassLoader {
     /** ArrayList containing jar indexes for various jars available to this classloader */
     private ArrayList<JarIndex> jarIndexes = new ArrayList<JarIndex>();
 
+    /** Set of classpath strings declared in the manifest.mf files */
+    private Set<String> classpaths = new HashSet<String>();
+
     /** File entries in the jar files available to this classloader */
     private TreeSet<String> jarEntries = new TreeSet<String>();
 
-    /** Map of specific codesources to securitydesc */
+    /** Map of specific original (remote) CodeSource Urls  to securitydesc */
     private HashMap<URL, SecurityDesc> jarLocationSecurityMap =
             new HashMap<URL, SecurityDesc>();
     
@@ -503,7 +510,7 @@ public class JNLPClassLoader extends URLClassLoader {
                     }
                 }
 
-                jarLocationSecurityMap.put(location, jarSecurity);
+                jarLocationSecurityMap.put(jarDesc.getLocation(), jarSecurity);
             } catch (MalformedURLException mfe) {
                 System.err.println(mfe.getMessage());
             }
@@ -725,7 +732,10 @@ public class JNLPClassLoader extends URLClassLoader {
 
                                     try {
                                         URL fileURL = new URL("file://" + extractedJarLocation);
-                                        addURL(fileURL);
+                                        // there is no remote URL for this, so lets fake one
+                                        URL fakeRemote = new URL(jar.getLocation().toString() + "!" + je.getName());
+                                        CachedJarFileCallback.getInstance().addMapping(fakeRemote, fileURL);
+                                        addURL(fakeRemote);
 
                                         SecurityDesc jarSecurity = file.getSecurity();
 
@@ -746,7 +756,7 @@ public class JNLPClassLoader extends URLClassLoader {
                                                     codebase.getHost());
                                         }
 
-                                        jarLocationSecurityMap.put(fileURL, jarSecurity);
+                                        jarLocationSecurityMap.put(fakeRemote, jarSecurity);
 
                                     } catch (MalformedURLException mfue) {
                                         if (JNLPRuntime.isDebug())
@@ -761,15 +771,25 @@ public class JNLPClassLoader extends URLClassLoader {
 
                         }
 
-                        addURL(location);
+                        addURL(jar.getLocation());
 
                         // there is currently no mechanism to cache files per
                         // instance.. so only index cached files
                         if (localFile != null) {
-                            JarIndex index = JarIndex.getJarIndex(new JarFile(localFile.getAbsolutePath()),
-                                                                  null);
+                            CachedJarFileCallback.getInstance().addMapping(jar.getLocation(), localFile.toURL());
+
+                            JarFile jarFile = new JarFile(localFile.getAbsolutePath());
+                            Manifest mf = jarFile.getManifest();
+
+                            if (file instanceof PluginBridge) {
+                                classpaths.addAll(getClassPathsFromManifest(mf, jar.getLocation().getPath()));
+                            }
+
+                            JarIndex index = JarIndex.getJarIndex(jarFile, null);
                             if (index != null)
                                 jarIndexes.add(index);
+                        } else {
+                            CachedJarFileCallback.getInstance().addMapping(jar.getLocation(), jar.getLocation());
                         }
 
                         if (JNLPRuntime.isDebug())
@@ -1008,8 +1028,30 @@ public class JNLPClassLoader extends URLClassLoader {
             try {
                 result = loadClassExt(name);
             } catch (ClassNotFoundException cnfe) {
+                // Not found in external loader either
 
-                // Not found in external loader either. As a last resort, look in any available indexes
+                // Look in 'Class-Path' as specified in the manifest file
+                try {
+                    for (String classpath: classpaths) {
+                        JARDesc desc;
+                        try {
+                            URL jarUrl = new URL(file.getCodeBase(), classpath);
+                            desc = new JARDesc(jarUrl, null, null, false, true, false, true);
+                        } catch (MalformedURLException mfe) {
+                            throw new ClassNotFoundException(name, mfe);
+                        }
+                        addNewJar(desc);
+                    }
+
+                    result = loadClassExt(name);
+                    return result;
+                } catch (ClassNotFoundException cnfe1) {
+                    if (JNLPRuntime.isDebug()) {
+                        cnfe1.printStackTrace();
+                    }
+                }
+
+                // As a last resort, look in any available indexes
 
                 // Currently this loads jars directly from the site. We cannot cache it because this
                 // call is initiated from within the applet, which does not have disk read/write permissions
@@ -1027,33 +1069,13 @@ public class JNLPClassLoader extends URLClassLoader {
                             } catch (MalformedURLException mfe) {
                                 throw new ClassNotFoundException(name);
                             }
-
-                            available.add(desc);
-
-                            tracker.addResource(desc.getLocation(),
-                                    desc.getVersion(),
-                                    null,
-                                    JNLPRuntime.getDefaultUpdatePolicy()
-                                    );
-
-                            URL remoteURL;
                             try {
-                                remoteURL = new URL(file.getCodeBase() + jarName);
-                            } catch (MalformedURLException mfe) {
-                                throw new ClassNotFoundException(name);
-                            }
-
-                            URL u;
-
-                            try {
-                                u = tracker.getCacheURL(remoteURL);
+                                addNewJar(desc);
                             } catch (Exception e) {
-                                throw new ClassNotFoundException(name);
+                                if (JNLPRuntime.isDebug()) {
+                                    e.printStackTrace();
+                                }
                             }
-
-                            if (u != null)
-                                addURL(u);
-
                         }
 
                         // If it still fails, let it error out
@@ -1070,6 +1092,91 @@ public class JNLPClassLoader extends URLClassLoader {
         return result;
     }
 
+    /**
+     * Adds a new JARDesc into this classloader.
+     * <p>
+     * This will add the JARDesc into the resourceTracker and block until it
+     * is downloaded.
+     * @param desc the JARDesc for the new jar
+     */
+    private void addNewJar(final JARDesc desc) {
+
+        available.add(desc);
+
+        tracker.addResource(desc.getLocation(),
+                desc.getVersion(),
+                null,
+                JNLPRuntime.getDefaultUpdatePolicy()
+                );
+
+        // Give read permissions to the cached jar file
+        AccessController.doPrivileged(new PrivilegedAction<Void>() {
+            public Void run() {
+                Permission p = CacheUtil.getReadPermission(desc.getLocation(),
+                        desc.getVersion());
+
+                resourcePermissions.add(p);
+
+                return null;
+            }
+        });
+
+        final URL remoteURL = desc.getLocation();
+        final URL cachedUrl = tracker.getCacheURL(remoteURL); // blocks till download
+
+        available.remove(desc); // Resource downloaded. Remove from available list.
+        
+        try {
+
+            // Verify if needed
+
+            final JarSigner signer = new JarSigner();
+            final List<JARDesc> jars = new ArrayList<JARDesc>();
+            jars.add(desc);
+
+            // Decide what level of security this jar should have
+            // The verification and security setting functions rely on 
+            // having AllPermissions as those actions normally happen
+            // during initialization. We therefore need to do those 
+            // actions as privileged.
+
+            AccessController.doPrivileged(new PrivilegedExceptionAction<Void>() {
+                public Void run() throws Exception {
+                    signer.verifyJars(jars, tracker);
+
+                    if (signer.anyJarsSigned() && !signer.getAlreadyTrustPublisher()) {
+                        checkTrustWithUser(signer);
+                    }
+
+                    final SecurityDesc security;
+                    if (signer.anyJarsSigned()) {
+                        security = new SecurityDesc(file,
+                                SecurityDesc.ALL_PERMISSIONS,
+                                file.getCodeBase().getHost());
+                    } else {
+                        security = new SecurityDesc(file,
+                                SecurityDesc.SANDBOX_PERMISSIONS,
+                                file.getCodeBase().getHost());
+                    }
+
+                    jarLocationSecurityMap.put(remoteURL, security);
+
+                    return null;
+                }
+            });
+
+            addURL(remoteURL);
+            CachedJarFileCallback.getInstance().addMapping(remoteURL, cachedUrl);
+
+        } catch (Exception e) {
+            // Do nothing. This code is called by loadClass which cannot 
+            // throw additional exceptions. So instead, just ignore it. 
+            // Exception => jar will not get added to classpath, which will 
+            // result in CNFE from loadClass.
+            e.printStackTrace();
+        }
+    }
+
     /**
      * Find the class in this loader or any of its extension loaders.
      */
@@ -1260,7 +1367,7 @@ public class JNLPClassLoader extends URLClassLoader {
     /**
      * Returns the security descriptor for given code source URL
      *
-     * @param source The code source
+     * @param source the origin (remote) url of the code
      * @return The SecurityDescriptor for that source
      */
 
@@ -1325,6 +1432,44 @@ public class JNLPClassLoader extends URLClassLoader {
         return file.getDownloadOptionsForJar(jar);
     }
 
+    /**
+     * Returns a set of paths that indicate the Class-Path entries in the
+     * manifest file. The paths are rooted in the same directory as the
+     * originalJarPath.
+     * @param mf the manifest
+     * @param originalJarPath the remote/original path of the jar containing
+     * the manifest
+     * @return a Set of String where each string is a path to the jar on
+     * the original jar's classpath.
+     */
+    private Set<String> getClassPathsFromManifest(Manifest mf, String originalJarPath) {
+        Set<String> result = new HashSet<String>();
+        if (mf != null) {
+            // extract the Class-Path entries from the manifest and split them
+            String classpath = mf.getMainAttributes().getValue("Class-Path");
+            if (classpath == null || classpath.trim().length() == 0) {
+                return result;
+            }
+            String[] paths = classpath.split(" +");
+            for (String path : paths) {
+                if (path.trim().length() == 0) {
+                    continue;
+                }
+                // we want to search for jars in the same subdir on the server
+                // as the original jar that contains the manifest file, so find
+                // out its subdirectory and use that as the dir
+                String dir = "";
+                int lastSlash = originalJarPath.lastIndexOf("/");
+                if (lastSlash != -1) {
+                    dir = originalJarPath.substring(0, lastSlash + 1);
+                }
+                String fullPath = dir + path;
+                result.add(fullPath);
+            }
+        }
+        return result;
+    }
+
     /*
      * Helper class to expose protected URLClassLoader methods.
      */
