@@ -160,6 +160,12 @@ gchar* out_pipe_name;
 // Applet viewer output watch source.
 gint out_watch_source;
 
+// Thread ID of plug-in thread
+pthread_t itnp_plugin_thread_id;
+
+// Mutex to lock async call queue
+pthread_mutex_t pluginAsyncCallMutex;
+
 // Applet viewer output channel.
 GIOChannel* out_to_appletviewer;
 
@@ -1169,26 +1175,7 @@ void consume_message(gchar* message) {
            data = (ITNPPluginData*) instance->pdata;
         }
 
-      if (g_str_has_prefix (parts[2], "url"))
-        {
-          // Open the URL in a new browser window.
-          gchar* decoded_url = (gchar*) calloc(strlen(parts[3]) + 1, sizeof(gchar));
-          IcedTeaPluginUtilities::decodeURL(parts[3], &decoded_url);
-
-          PLUGIN_DEBUG ("plugin_in_pipe_callback: opening URL %s\n", decoded_url);
-          PLUGIN_DEBUG ("plugin_in_pipe_callback: URL target %s\n", parts[4]);
-
-          NPError np_error =
-            (*browser_functions.geturl) (data->owner, decoded_url, parts[4]);
-
-
-          if (np_error != NPERR_NO_ERROR)
-            PLUGIN_ERROR ("Failed to load URL.");
-
-          g_free(decoded_url);
-          decoded_url = NULL;
-        }
-      else if (g_str_has_prefix (parts[2], "status"))
+      if (g_str_has_prefix (parts[2], "status"))
         {
 
           // clear the "instance X status" parts
@@ -2223,8 +2210,6 @@ NP_Initialize (NPNetscapeFuncs* browserTable, NPPluginFuncs* pluginTable)
 
   PLUGIN_DEBUG ("NP_Initialize: using %s\n", appletviewer_executable);
 
-  PLUGIN_DEBUG ("NP_Initialize return\n");
-
   plugin_req_proc = new PluginRequestProcessor();
   java_req_proc = new JavaMessageSender();
 
@@ -2238,6 +2223,16 @@ NP_Initialize (NPNetscapeFuncs* browserTable, NPPluginFuncs* pluginTable)
   pthread_create (&plugin_request_processor_thread2, NULL, &queue_processor, (void*) plugin_req_proc);
   pthread_create (&plugin_request_processor_thread3, NULL, &queue_processor, (void*) plugin_req_proc);
 
+  itnp_plugin_thread_id = pthread_self();
+
+  pthread_mutexattr_t attribute;
+  pthread_mutexattr_init(&attribute);
+  pthread_mutexattr_settype(&attribute, PTHREAD_MUTEX_RECURSIVE);
+  pthread_mutex_init(&pluginAsyncCallMutex, &attribute);
+  pthread_mutexattr_destroy(&attribute);
+
+  PLUGIN_DEBUG ("NP_Initialize return\n");
+
   return NPERR_NO_ERROR;
 
  cleanup_appletviewer_executable:
@@ -2382,6 +2377,9 @@ NP_Shutdown (void)
   g_free (in_pipe_name);
   in_pipe_name = NULL;
 
+  // Destroy the call queue mutex
+  pthread_mutex_destroy(&pluginAsyncCallMutex);
+
   initialized = false;
 
   pthread_cancel(plugin_request_processor_thread1);
