@@ -912,8 +912,8 @@ public class PluginAppletViewer extends XEmbeddedFrame
      */
     public void showDocument(URL url, String target) {
         try {
-            // FIXME: change to postCallRequest
-            write("url " + UrlUtil.encode(url.toString(), "UTF-8") + " " + target);
+            Long reference = getRequestIdentifier();
+            write("reference " + reference +  " LoadURL " + UrlUtil.encode(url.toString(), "UTF-8") + " " + target);
         } catch (IOException exception) {
             // Deliberately ignore IOException.  showDocument may be
             // called from threads other than the main thread after
@@ -1609,10 +1609,15 @@ public class PluginAppletViewer extends XEmbeddedFrame
 
                 appletShutdown(p);
                 appletPanels.removeElement(p);
-                dispose();
 
-                if (tg.activeCount() > 0)
-                    tg.stop();
+                try {
+                    SwingUtilities.invokeAndWait(new Runnable() {
+                        public void run() {
+                            dispose();
+                        }
+                    });
+                } catch (Exception e) { // ignore, we are just disposing it
+                }
 
                 if (countApplets() == 0) {
                     appletSystemExit();
